# NT.ph is required for the predefined routines such as
# &EVENTLOG_SEQUENTIAL_READ.

require "NT.ph";

# required for the date conversion functions

use Time::Local;

use OLE;

# getStrDate returns the string version of a time based on the
# number of seconds since 1970.

sub getStrDate {
    my $epoch = shift @_;
    my ($tSec,$tMin,$tHour,$tMDay,$tMon,$tYear,$wday,$yday,$isdst) = localtime($epoch);
    return("$tHour:$tMin:$tSec $tMDay/",++$tMon,"/$tYear");
}

# getEpochDate is the reverse of the previous function in that it
# takes a string date, and converts it to an epoch value. This is
# used to make time comparisons easier

sub getEpochDate {
    $string  = shift @_;
    $string =~ /(\d{2}):(\d{2}):(\d{2})\s(\d{2})\/(\d{2})\/(\d{2})/;
    $time = Time::Local::timelocal($3,$2,$1,$4,$5,$6);
    return($time);
}

# this is where all of the work is done. After shifting all of the
# arguments into their appropriate variables, the function checks
# too see whether the log source is one of the three standard, or
# a backup file and opens it up accordingly. Note that the %search
# hash is passed in by reference.

sub searchEvents {
    my $source = shift @_;
    my $sHashRef = shift @_;
    my $delimiter = shift @_;
    my $output = shift @_;
    my %search = %$sHashRef;

    if($output eq 'excel') {
        $excel = CreateObject OLE 'Excel.Application.5' or die "unable to open Excel";
        $excel->Workbooks->Open('D:\perl\scripts\out1.xls') or die "unable to open Excel workbook";
    } else {
        open(OUTPUT,">>$output") or die "unable to open $output";
        select(OUTPUT);
    }


    if($source ne 'application' and $source ne 'system'
        and $source ne 'security') {
        Win32::OpenBackupEventLog($openlog,$ENV{COMPUTERNAME},$source) 
            or die "unable to connect to backup log $source";
    } else {
        Win32::OpenEventLog($openlog,$ENV{COMPUTERNAME},$source) 
            or die "unable to open $source log";
    }
    my $flag = &EVENTLOG_FORWARDS_READ | &EVENTLOG_SEQUENTIAL_READ;

    # this is the loop where each of the events in a log is
    # compared against the arguments provided. Notice that
    # those variables we are concerned with are placed in the
    # %event hash. This structure can later be easily compared
    # with the %search hash passed into the function.

    $i = 1;

    while ( Win32::ReadEventLog($openlog, $flag, 0, $evtHeader, $event{'source'},
                               $event{'computer'}, $event{'sid'}, $event{'data'},
                               $event{'strings'} )) {

        # this very long line is used to unpack the header information into its
        # respective variables. Note again the use of the %event hash.

        ($length, $reserved, $event{'recnum'}, $event{'tgen'}, $event{'twrite'},
         $event{'id'}, $event{'type'}, $nStrings, $event{'category'}, $rFlags,
         $clRecNum, $strOff, $sidLen, $sidOff, $dataLen, $dataOff) 
             = unpack "L16",$evtHeader;

        # since user information is passed as an sid, we must convert it
        # to a readable format. This is done with LookupAccountSID. In 
        # this case, the PDC is hard-wired into the function call.

        Win32::LookupAccountSID('iago', $event{'sid'}, $event{'account'}, $domain, $sidtype);

        # This is the loop which compares each of the keys in the %search hash
        # to their respective keys in the %event hash created from the event
        # just read from the log. Since the %search hash may not include all 
        # keys, there is no need to loop through every possible key. If a key
        # exists in the %search hash, and its value does not equal the
        # corresponding key in the %event hash, the loop is 
        # immediately stopped, and the event not added to the output.

        KEY: foreach $key(keys %search) {


            # the time matching is based in event generation

            if ($key eq 'begintime' or $key eq 'endtime') {
                if($event{'tgen'} >= $search{'begintime'} 
                   || $event{'tgen'} <= $search{'begintime'})
                {
                    $match = 1;
                    next;
                } else {
                    $match = 0;
                    last;
                }
            } elsif($key eq 'strings') {

                # in this example it is sufficient to match any word
                # in the strings key. The strings key is also known
                # as the description in the event viewer (minus the
                # built in strings added by the source).

                @words = split("\s+",$event{'strings'});
                foreach $word(@words) {
                    if($word eq $search{'strings'}) {
                        $match = 1;
                    } else {
                        $match = 0;
                    }
                }
                last KEY if not $match;

            # for all other keys, a direct mapping can be done

            } elsif ($event{$key} eq $search{$key}) {
                $match = 1;
                next;
            } else {
                $match = 0;
                last;
            }
        }
        
        # if the event has gotten this far without a mismatch,
        # it is sent to the output.
        if($output eq 'excel') {
            $j = 1;
            foreach $key(sort keys %event) {
                $excel->Workbooks(1)->Worksheets('Sheet1')->Cells($j++,$i)->{Value} = $event{$key};
            }
        } else {
            foreach $key(sort keys %event) {
                $output .= $event{$key}.$delimiter;
            }
            chop($output); 
            print $output,"\n" if $match;
            $output = "";
        }

        $i++;
    }
    if($output eq 'excel') {
        $excel->Save();
        $excel->Quit();
    } else {
        close($output);
    }
    Win32::CloseEventLog($log);
}

# a default delimiter is selected as space

$del = ' ';

# this is the loop where all of the command line arguments are taken
# and use to build up the options and search hash

while($command = shift @ARGV) {
    if($command eq '/m') {
        $key = shift @ARGV;
        $value = shift @ARGV;
        $match{$key} = $value;
    } elsif($command eq '/l') {
        $log = shift @ARGV;
    } elsif($command eq '/f') {
        $out = shift @ARGV;
    } elsif($command eq '/i') {

        # this option takes the input file, and after splitting each
        # line, adds the arguments to the @ARGV array
  
        $file = shift @ARGV;
        open(INPUT,$file) or die "cannot open $file for input";
        while(<INPUT>) {
            chomp;
            @newargs = split('\s+',$_);
            @ARGV = (@ARGV,@newargs);
        }
    } elsif($command eq '/t') {

        # this option sets the time limits to search for

        $arg = shift @ARGV;
        if( $arg eq 'after') {
            $str = shift @ARGV;
            $match{'begintime'} = getEpochTime($str);
            $match{'endtime'} = time;
        } elsif( $arg eq 'before') {
            $str = shift @ARGV;
            $match{'endtime'} = getEpochTime($str);
            $match{'begintime'} = 0;
        } else {
            $beginstr = shift @ARGV;
            $endstr = shift @ARGV;
            $match{'begintime'} = getEpochTime($beginstr);
            $match{'endtime'} = getEpochTime($endstr);
        }
    } elsif($command eq '/d') {

        # this command is used to set the delimiter to a comma for csv

        $arg = shift @ARGV;
        if($arg eq 'csv') {
            $del = ',';
        }
    } elsif($command eq '/a') {

        # this is used to run the search command
        # before loading up the next match variables
        
        searchEvents($log,\%match,$del,$out);
    }
        
}

# this runs the search for the last set of variable,
# or if there is only one set given

searchEvents($log,\%match,$del,$out);
