# this will scan a disk/directory, and look for all of the files to which a user
# has access in that tree. It finishes by dumping the information into a file
# called out.log

use Win32::UserAdmin;
use Win32::FileSecurity;

# The GetUserGroups function takes a user name, and returns all of the local
# and global groups to which that user belongs.

sub GetUserGroups {
    my $server = shift @_;
    my $username = shift @_;
    Win32::UserAdmin::UserGetLocalGroups($server, $username, \@localgroups);
    Win32::UserAdmin::UserGetGroups($server, $username, \@globalgroups);
    @groups = (@globalgroups, @localgroups);
}

# The RecurseDir function is where all the work is done. It takes a
# directory and hash of user and group names, recurses down through
# the directory, checking for the existence of any of the $usergroups
# entries in a file's ACL. If the file has an entry, it is added to the
# %list hash, which is returned at the end.

sub RecurseDir {
    my ($dir,$usergroups) = @_;
    my ($file,$fullname);
    my %list = ();
    opendir (DIR, "$dir");
    my @files = readdir DIR;
    closedir DIR;
    foreach $file (@files) {
        # don't act on . and ..
        next if ($file eq '.' || $file eq '..');
        $fullname = "$dir/$file";
        if (-d $fullname) {
            Win32::FileSecurity::Get($fullname, \%ACL);
            foreach $ACE(keys %ACL) {
                print "$fullname:$ACE\n";
                if(exists $usergroups{$ACE} or $ACE eq "Everyone") {
                    Win32::FileSecurity::EnumerateRights($ACL{$ACE}, \@rights);
                    $list{$fullname} = join("\t", @rights);
                    my %list2 = &RecurseDir($fullname,$usergroups);
                    foreach $key(keys %list2) {
                        $list{$key} = $list2{$key};
                    }
                }
            }
        } elsif (-f $fullname) {
            Win32::FileSecurity::Get($fullname, \%ACL);
            foreach $ACE(keys %ACL) {
                if(exists $usergroups{$ACE} or $ACE eq "Everyone") {
                    Win32::FileSecurity::EnumerateRights($ACL{$ACE}, \@rights);
                    $list{$fullname} = join("\t", @rights);
                }
            }
        }
    }
    return( %list);
}

# get username from command line

$user = $ARGV[0];
print "Finding ACL entries for $user in $ARGV[1]\n";
Win32::UserAdmin::GetDomainController('','',$server);

# first, we need to find out all the groups to which the user belongs
# and put it in a hash table. We also place the username in the table

@usergroups = GetUserGroups($server, $user);
foreach $usergroup(@usergroups) {
    $usergroups{$usergroup} = " ";
}
$usergroups{$user} = " ";

# finally, we open an output file, and set the &RecurseDir function running,
# outputting the results into the out.log file.

open(OUT, ">out.log") or die "Cannot open output";
%files = RecurseDir($ARGV[1],\@usergroups);
foreach $file(keys %files) {
    print OUT $file,"\t",$files{$file},"\n";
}
